"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.format = exports.convert = exports.applyMixins = exports.guid = void 0;
let number = 0;
const guid = () => {
    return number++;
};
exports.guid = guid;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function applyMixins(derivedCtor, constructors) {
    constructors.forEach((baseCtor) => {
        Object.getOwnPropertyNames(baseCtor.prototype).forEach((name) => {
            if (name === 'constructor')
                return;
            Object.defineProperty(derivedCtor.prototype, name, Object.getOwnPropertyDescriptor(baseCtor.prototype, name) || Object.create(null));
        });
    });
}
exports.applyMixins = applyMixins;
/**
 * escapeHTML
 */
function convert(str) {
    if (typeof str !== 'string')
        return str;
    let result = '';
    let escape = false;
    for (let i = 0; i < str.length; i++) {
        const c = str.charAt(i);
        let cstr = '';
        if ((c >= ' ' && c <= '~') || c === '\r' || c === '\n') {
            if (c === '&') {
                cstr = '&amp;';
                escape = true;
            }
            else if (c === '"') {
                cstr = '&quot;';
                escape = true;
            }
            else if (c === '<') {
                cstr = '&lt;';
                escape = true;
            }
            else if (c === '>') {
                cstr = '&gt;';
                escape = true;
            }
            else {
                cstr = c.toString();
            }
        }
        else {
            cstr = '&#' + c.charCodeAt(0).toString() + ';';
        }
        result = result + cstr;
    }
    return escape ? result : str;
}
exports.convert = convert;
const format = (value) => {
    if (Array.isArray(value)) {
        return '[' + value.map(exports.format.bind(this)).join(', ') + ']';
    }
    if (typeof value === 'object' && value !== null) {
        if (value.toString.toString().indexOf('[native code]') === -1) {
            return value;
        }
        const kvJoin = (k) => `${k}=${(0, exports.format)(value[k])}`;
        return '{' + Object.keys(value).map(kvJoin).join(', ') + '}';
    }
    return value;
};
exports.format = format;
//# sourceMappingURL=utils.js.map